      FUNCTION CSTART(R,celln,inmode) 
        real*8 R(3),CSTART                                               ! i
        character*12 inmode                                              ! i
        integer*4 celln                                                  ! i
        real*8 C0                                                        ! a 
**************************************************************************
*               CUSTOMIZE THIS FUNCTION TO FIT YOUR PROBLEM              *
*                      USAGE:  3D diffusion problems                     *
* The starting Concentration / Temperature may depend on the coordinates *
*         e.g.  C0 = 273                                                 *
*               C0 = 273 * ( R(1) + R(2) )                               *
*         It can also depend on the material (celln), or on both         *
*     Variables:                                                         *
*        - R(3): absolute coordinates, x,y,z [cm] of the volume boxel    *
*        - CSTART: Starting concentration / Temperature                  *
*        - celln: number of the cell                                     *
*        - inmode: (5) ==> mass diffusion problem (6) ==> heat transfer  *
**************************************************************************
        IF (celln.ge.1) THEN  
          IF (inmode.eq.'5') THEN
            C0 = 1.0
          ELSE IF (inmode.eq.'6') THEN
            C0 = 500.0 ! starting temperature
          END IF
        ELSE                  ! outer space
          IF (inmode.eq.'5') THEN
            C0 = 0.0
          ELSE IF (inmode.eq.'6') THEN
            C0 = 298.0
          END IF
        END IF
        CSTART = C0
      END


      FUNCTION gener(R,C0,t,PAR,Dbq,ncell,celln,inmode) 
        integer*4 ncell,celln                                            ! i
        character*12 inmode                                              ! i
        real*8 R(3),C0,t,PAR(5),Dbq(7,ncell)                             ! i
        real*8 dV,CpRho                                                  ! a  
        real*8 gener                                                     ! o
**************************************************************************
*      Customize this function to include heat or mass sources           *
*        that depend on the position X, or/and time, or/and              *
*             the concentration, celln, emissivity, ...                  *
**************************************************************************
*   VARIABLES:                                                           *
*        R(3) = coordinates (x,y,z)[cm]           DO NOT MODIFY VALUE!   *
*          C0 = Concentration [par/cm^3] | Temperature [K]               *
*           t = elapsed time [s]                                         *
*      PAR(1) = dV = dx * dy * dz [cm^3]  Useful to normalize generation *
*      PAR(2) = concentration/temperature at point X  [cm^-3 | T]        *
*      PAR(3) = celln                                                    *
*      PAR(4) = t_desorption/emissivity [s | W/cm^2*K^4]                 *
*      PAR(5) = dt [s]                                                   *
*      inmode = 5 mass diffusion | 6 heat transfer                       *
*    ------------------------------------------------------              *
*      Dbq(1) =  D0 [cm^2/K^0 s] | k0  [W/cm K^1]                        *
*      Dbq(2) =  DC [cm^2/K^1 s] | kT  [W/cm K^2]                        * 
*      Dbq(3) = DCC [cm^2/k^2 s] | kTT [W/cm K^3]                        *
*      Dbq(4) =          t_s [s] | Eps [W/K^4]                           *
*      Dbq(5) =               -  | Cp0*rho  [W s/cm3 K^1]                *
*      Dbq(6) =               -  | CpT*rho  [W s/cm3 K^2]                * 
*      Dbq(7) =               -  | CpTT*rho [W s/cm3 K^3]                *
*       ncell = number of cells of the geometry                          *
*........................................................................*
*       CpRho = Cp*Rho [W s/K cm^3]                                      * 
**************************************************************************
*   UNITS of gener                                gener                  *
*      For MASS TRANSFER:              dV                                *
*        need : par / (cm^3 * s) --------------> par / s                 *
*      For HEAT TRANSFER:          (dV/CpRho)                            *    
*        need :   J / (cm^3 * s) -------------->   K / s                 *
*   EXAMPLES:                                                            *
*      1) constant generation of particles, 1E5 (par/cm3 s)              *
*             gener [par/s] = 1E5 * dV                                   *
*      2) constant heat generation: (1kW/cm^3)                           *
*        ===> gener [K/s]   = 1000 * dV / (CpRho)                        *
**************************************************************************
*   NOTE:                                                                *
*        The generation may depend on time and on X, ...                 *
**************************************************************************
        dV = PAR(1)  ! [cm^3]
        gener = 0.0
        IF (inmode.eq.'5') THEN      ! 3D mass diffusion      
*         gener = 1E4 * dV                ! [par/s]
        ELSE IF (inmode.eq.'6') THEN ! 3D heat transfer
          CpRho = (Dbq(5,celln))
     $          + (Dbq(6,celln) * T)
     $          + (Dbq(7,celln) * T * T)
          IF (celln.eq.1) THEN
* EXAMPLE 10 kW / cm^3 deposited in cell = 1:
            gener = 10000.0 / CpRho  ! [W/cm^3]-->[K/s]
          END IF
        END IF
      END 


      FUNCTION Dijkt(I,X,t,N,C,Dbq,ncell,inmode)
        integer*4 N(3),I(3),ncell                                        ! i
        character*12 inmode
        real*8 C(300,300,300,2),X(3),Dbq(7,ncell),t
        integer*4 celln                                                  ! a
        real*8 CorT,CpRho,k                              
        real*8 Dijkt                                                     ! o
**************************************************************************
*               CUSTOMIZE THIS FUNCTION TO FIT YOUR PROBLEM              *
**************************************************************************
*                         written by MSL, 2006                           *
**************************************************************************
*     This function computes the DIFFUSION coefficient | CONDUCTIVITY    *
*     for a given boxel (that corresponds to position X), and depending  *
*     on the CONCENTRATION | TEMPERATURE, parsed constants Dbq and cell  *
*     number                                                             *
*   VARIABLES:                                                           *
*    case         (MASS) transfer             (HEAT) transfer            *
*   ---------------------------------------------------------------      *
*   inmode:           = 5               |            =6                  *
*    CpRho:                             |  Cp*Rho [W s/K cm^3]           *
*        k:                             |  conductivity [W/K cm]         * 
*    Dijkt: Diffusion coeff.(D)[cm^2/s] |  k/(Cp*Rho) [cm^2/s]           *
*     C...:    {concentration,celln}    |   {Temperature,celln}          *
*      Dbq:     diffusion constants     |  conductance/emiss. (Cond.dat) *
*        t:                      elapsed time [s]                        *
*     N(3):              number of divisions in {x,y,z}                  *
*     I(3):                 boxel identifier {i,j,k}                     *
*     X(3):            position of center of boxel {x,y,z}               *
*    ncell:            total number of regions in geometry               *
*   ---------------------------------------------------------------      * 
*   NOTES:                                                               *
*     A quadratic law D(C) and k(T) are assumed, but other laws can be   *
*     implemented in this function                                       * 
**************************************************************************
*     The user may introduce fancy features now, for example:            *
*     1) Time dependency of diffusion coefficient                        *
*        Dijkt = ... * sin(t)                                            *
*     2) X-dependency of diffusion coefficient                           *
*     ...                                                                *
************************************************************************** 
        celln = NINT(C(I(1),I(2),I(3),2))
        IF (inmode.eq.'5') THEN
          IF (celln.eq.0) THEN
          Dijkt = 0.0
          ELSE
           CorT = C(I(1),I(2),I(3),1)  ! Concentration 
          Dijkt = (Dbq(1,celln))
     $          + (Dbq(2,celln) * CorT)
     $          + (Dbq(3,celln) * CorT * CorT)
          END IF
        ELSE IF (inmode.eq.'6') THEN
          IF (celln.eq.0) THEN
          Dijkt = 0.0
          ELSE
           CorT = C(I(1),I(2),I(3),1)  ! Temperature 
              k = (Dbq(1,celln))
     $          + (Dbq(2,celln) * CorT)
     $          + (Dbq(3,celln) * CorT * CorT)
          CpRho = (Dbq(5,celln))
     $          + (Dbq(6,celln) * CorT)
     $          + (Dbq(7,celln) * CorT * CorT)
          Dijkt = k / CpRho
          END IF
        END IF
      END


      FUNCTION radia(i,M,T,C,N,R,PAR,Dbq,dA,ncell) 
        integer*4 i,ncell,N(3)                                           ! i
        real*8 M(3),T,PAR(5),dA(3),Dbq(7,ncell),C(300,300,300,2),R(3)    ! i
        real*8 emissivity,sigSB,eps,T0,dV,radia,CpRho,norm               ! a
        integer*4 celln                                                  ! a 
**************************************************************************
*    THIS SUBROUTINE checks if we are in a heat radiation problem        *
*      (inmode=6) and, if so, it computes the radiated power Qr [W]      *
*      as a function of the emittance (eps), the exposed area dA(i)      *
*      and the temperature difference (T=C0,T0=external)                 *
*    VARIABLES:                                                          *
*           i = exposed area normal vector {i,j,k}                       *
*        M(3) = material of neighbouring cell (should be vacuum, M=0)    *
*           T = Temperature of the boxel [K]                             *
*         C...= Temperature{i,j,k,celln}                                 *
*        N(3) = number of divisions in {x,y,z}                           *
*        X(3) = position of center of boxel {x,y,z}                      *
*      PAR(1) = dV = dx * dy * dz [cm^3]  Useful to normalize generation *
*      PAR(2) = concentration/temperature at point X  [cm^-3 | T]        *
*      PAR(3) = celln                                                    *
*      PAR(4) = t_desorption/emissivity [s | W/cm^2*K^4]                 *
*      PAR(5) = dt [s]                                                   *
*    ------------------------------------------------------              *
*      Dbq(1) =  D0 [cm^2/K^0 s] | k0  [W/cm K^1]                        *
*      Dbq(2) =  DC [cm^2/K^1 s] | kT  [W/cm K^2]                        * 
*      Dbq(3) = DCC [cm^2/k^2 s] | kTT [W/cm K^3]                        *
*      Dbq(4) =          t_s [s] | Eps [W/K^4]                           *
*      Dbq(5) =               -  | Cp0*rho  [W s/cm3 K^1]                *
*      Dbq(6) =               -  | CpT*rho  [W s/cm3 K^2]                * 
*      Dbq(7) =               -  | CpTT*rho [W s/cm3 K^3]                *
*........................................................................*
*       dA(3) = dA(i) is the perpendicular area for normal i [cm^2]      *
*       CpRho = Cp*Rho [W s/K cm^3]                                      *
*          T0 = ambient temperature [K]                                  *
*       sigSB = Stephan-Boltzmann constant = 5.6703E-12 [W/cm^2*K^4]     *
**************************************************************************
*      CUSTOMIZATION                                                     *
*        - replace eps by a function of temperature eps(T)               *
*        - replace T0 by the ambient temperature or by the temperature   *
*             of enfolding cells, e.g T0=C(10,4,9,1)                     *
**************************************************************************
        celln = NINT(PAR(3))                
        IF (M(i).lt.1E-14) THEN
          CpRho = (Dbq(5,celln))
     $          + (Dbq(6,celln) * T)
     $          + (Dbq(7,celln) * T * T)
          dV  = PAR(1)                                                   ! [cm3]
          sigSB = 5.6703E-12                                             ! [W/K^4 cm^2]
          emissivity = PAR(4)                                            ! [-]
          eps = sigSB * emissivity                                       ! [W/K^4 cm^2]
          T0  = 298.0                                                    ! [K] ambient temperature 
**************************************************************************
*      EXAMPLE: heat3D.t                                                 *
*      - The solid W sphere sees the inner side of the outer Ta spheric  *
*        shell. C(10,90,90,1) corresponds to the temperature in a point  *
*        of the Ta inner side. The spheric symmetry allows to leave that *
*        same reference point for whatever position                      *
*      - The Ta spheric shell (celln=2) sees, in the inside, the W       *
*        sphere C(31,90,90,1) and, in the outside, the open air, T=298   *
**************************************************************************
          IF (celln.eq.1) THEN       ! W shpere
            T0  = C(10,90,90,1)         ! Temperature of inner face of Ta shell
          ELSE IF (celln.eq.2) THEN  ! Ta shell
            IF (norm(R).le.4.40) THEN   ! inner face
            T0  = C(31,90,90,1)            ! ==> ~ Surface temperature of the W sphere
            ELSE                        ! outer face  
            T0 = 298.0                     ! ==> Sky temperature
            END IF
          END IF
          radia = eps * ((T0**4)-(T**4)) * dA(i) / (CpRho*dV)
**************************************************************************
*      EXAMPLE 2   CONVECTION                                            *
*      - Note that you can also include convection                       *
*        First define the variable "conv" and the paramter "constant"    *
*      - Uncomment these lines:                                          *
*          conv = constant * (TO - T)                                    *
*          radia = eps * ((T0**4)-(T**4)) * dA(i) / (CpRho*dV) + conv    *
**************************************************************************
        ELSE
          radia = 0.0
        END IF
      END


      SUBROUTINE userPRINT3D(t,N,C,C0,nbin,Cmax,Imax,Rmax,cellmax,
     $       Cmin,Imin,Rmin,cellmin,sum,sum0)
        integer*4 N(3),Imax(3),Imin(3),cellmax,cellmin                   ! i
        real*8 C(300,300,300,2),C0(300,300,300,2),Cmax,Cmin,nbin         ! i
        real*8 t,sum,sum0                                                ! i
        real*8 Rmax(3),Rmin(3),fmax,fmin,Cav,fav                         ! i
        integer*4 i                                                      ! a
        logical fileNOTyetOPEN                                           ! a
**************************************************************************
*               CUSTOMIZE THIS FUNCTION TO FIT YOUR PROBLEM              *
**************************************************************************
*                         written by MSL, 2006                           *
**************************************************************************
*   VARIABLES:                                                           *
*      - t:     Elapsed time [s]                                         *
*      - N(3):  Number of bins in each direction x,y,z                   *
*      - C(300,300,300,2): two values for each voxel                     *
*            C(300,300,300,=2): material index (region number)           *
*            C(300,300,300,=1): Concentration | Temperature              *
*      - C0(300,300,300,2): stores the values of C at t=0                *
*      - Cmax(Cmin): Highest(Lowest) Concentration | temperature         *
*      - Imax(Imin): Pointers {i,j,k} of the boxel with Cmax(Cmin)       *
*      - Rmax(Rmin): Coordinates of the boxel of Cmax(Cmin) {x,y,z} [cm] *
*      - cellmax(cellmin): the boxel of Cmax(Cmin) belongs to celln...   *
*      - nbin:  number of boxels belonging to a cell                     *
*      - sum:   sum of concentrations/temperatures at time t             *     
*      - sum0:  sum of concentrations/temperatures at time 0             *     
**************************************************************************
*   EXAMPLE 1
*      - Print the evolution of the matrix C (lot of memory!!):
**
*        DATA fileNOTyetOPEN / .TRUE. /
*        IF ( fileNOTyetOPEN) THEN
*        fileNOTyetOPEN = .FALSE.
*        OPEN(UNIT=26,FILE='26.out')
*        write(6,*)'Creating file unit 26'
*        END IF
**
*        write(6,*)C
**************************************************************************
* 
       END
