      SUBROUTINE CustomColl(X3,grad3,T,A,surn,celln,rc,ts,tde,U3,COL6)
       integer*4 surn,celln                                              ! i 
       real*8 X3(3),grad3(3),rc,T,A,ts                                   ! i
       real*8 U3(3),COL6(6),tde                                          ! i/o
       real*8 SU3(3),SV3(3),SW3(3)                                       ! a
       real*8 phi,alpha,cosphi,n,zero,k,V3scaV3,norm,a1,b1,c1,v,t0       ! a
       external rand
**************************************************************************
*                                                                        *
*     WRITTEN BY:                                                        *
*                        MARIO SANTANA LEITNER, 2006                     *
*                                                                        * 
*----------------------------------------------------------------------- *
*                                                                        *
*               CUSTOMIZE THIS FUNCTION TO FIT YOUR PROBLEM              *
*   USE:  transport problems with special physics for the collisions     *
*                                                                        *
*   HIGHLIGHTS:                                                          *
*           - customized law, as a function of impinging angle           *
*           - desorption time sampled from Frenkel Law                   *
*           - simulation of COLD spots                                   *
*           - desorption function of temperature (position)              *
*
*   ACTIVATE USE THROUGH option 'C' at RUNTIME to the QUESTION:          *
*     'What kind of treatment of the collisions?'                        *
*     'S=Specular(E<<),B=Knudsen-Lambert,D=Debye,C=Custom                *
*                                                                        *
*   VARIABLES:                                                           *
*             INPUT                                                      *
*     X3(3)  -------  Absolute position at collision point (x,y,z) [cm]  *
*     grad(3) ------  Surface gradient (normal) at collision point       *
*                         always pointing inwards and normalized.        *
*     T       ------  Starting energy of the atom [k] (source card)      *
*     A       ------  Atomic mass [uam] (source card of input file)      *
*     surn    ------  Surface number (from input file)                   *
*     celln   ------  Present number of cell (region)                    *
*     rc     -------  First parameter of the surface after surface       * 
*                         index number (e.g. roughness coefficient)      *
*     ts     -------  Average sticking time per collision (if it is ~    *
*                        constant all-over, it is normally define as 0   *
*                        in the runtime options. The output file then    *
*                        includes a sensitivity analysis for values      *
*                        other than 0.                                   *
*             I/OUT                                                      * 
*     U3(3)  -------  Incoming/Outgoing velocity, ux,uy,uz of the atom   *
*                        [m/s]                                           *
*     COL6(6) ------  Counter for the type of collision for each history *
*                         1: Maxwell type 1                              *
*                         2: Maxwell type 2                              * 
*                         3: Mirror-like                                 *
*                         4: Debye (semi-classic)                        * 
*                         5: Custom 1                                    * 
*                         6: Custom 1                                    *
*                                                                        * 
*     tde    -------  Particle desorption time [s]. Normally desorption  *
*                        time in the collisions is taken into account    *
*                        at the end of each history as "ts*COLL".        * 
*                        If you use a special desorption time distri-    *
*                        bution, (examples 4, 5), update the value of    *
*                        tde (as shown in those examples).               *  
*           AUXILIARY                                                    *
*     SU3(3),...----  Base of auxiliary vectors to build up reflection   *
*     phi,alpha...--  Auxiliary angles to build reflected direction      *
*     ...                                                                *
*            EXTERNAL                                                    *
*     rand   -------  Seed for random generation                         *
*                                                                        *
*   NOTES:                                                               *
*     - Check available mathematical functions in math.f                 *
*     - Surface ionization is done separately                            *
*     -                                                                  *
*                                                                        *
**************************************************************************
*
*
*------------------------------------------------------------------------*
*  EXAMPLE 1, snell reflection if rc>0.5, otherwise perpendicular        *
*------------------------------------------------------------------------* 
*       zero = 0. 0           
*       IF (rc.gt.0.5) THEN
*         k = V3scaV3(grad3,U3) ! k = grad3(1)*U3(1)+grad3(2)*U3(2)+grad3(3)*U3(3) < 0
*         U3(1) = U3(1) - 2 * k * grad3(1)  ! 
*         U3(2) = U3(2) - 2 * k * grad3(2)
*         U3(3) = U3(3) - 2 * k * grad3(3)
*         COL6(5) = COL6(5) + 1.0
*       ELSE
*         U3(1) = grad3(1)
*         U3(2) = grad3(2)
*         U3(3) = grad3(3)
*         COL6(6) = COL6(6) + 1.0
*       END IF
*
*------------------------------------------------------------------------*
*  EXAMPLE 2, If the incidence angle is within a cone of 60 \deg around  *
*      the surface normal, then reflect isotropically within that cone,  *
*      otherwise use mirror-like reflections                             *
*------------------------------------------------------------------------* 
*
*       cosphi = V3scaV3(U3,grad3)/(norm(U3)*norm(grad3))  ! cos(phi)=a * b / |a|*|b| < 0 
*       IF (cosphi.lt.0.5) THEN                            ! (phi>60\deg)
*         k = V3scaV3(grad3,U3) ! k = grad3(1)*U3(1)+grad3(2)*U3(2)+grad3(3)*U3(3)
*         U3(1) = U3(1) - 2 * k * grad3(1)    
*         U3(2) = U3(2) - 2 * k * grad3(2)
*         U3(3) = U3(3) - 2 * k * grad3(3)
*         COL6(5) = COL6(5) + 1.0
*       ELSE                                               ! (|phi|>60\deg)
*         CALL isotropic(grad3,U3,60)                      ! defined in math.f
*         COL6(6) = COL6(6) + 1.0
*       END IF
*
*------------------------------------------------------------------------*
*  EXAMPLE 3, Poth (or Phong) model.                                     *
*      The particle is reflected with a cosine^n law around the          *
*      mirror-like reflected direction.                                  * 
*      n depends on the roughness                                        *
*------------------------------------------------------------------------* 
*
*       zero=0.0
*       n = exp(rc*5.887)                                  ! rc-->0 ==> very rough
*       cosphi = V3scaV3(U3,grad3)/(norm(U3)*norm(grad3))  ! cos(phi)=a * b / |a|*|b| < 0 
*C- - - - - Sample the alpha from "cos(alpha)**n" by REJECTION 
*       alpha = asin(rand(zero)*cosphi)                    ! alpha uniform between 0 and 90-phi
*       DO WHILE (rand(zero).ge.(cos(alpha))**(n-1.0))
*         alpha = asin(rand(zero)*cosphi)
*       END DO
*C- - - - - Now sample the azimuthal angle, beta by INVERSE transformation
*       beta=rand(zero)*2*3.14159265
*C- - - - - Now build up local base and get exit vector U3
*       CALL surfaceBase(SU3,SV3,grad3)
*       a1 = SIN(alpha) * COS(beta)
*       b1 = SIN(alpha) * SIN(beta)
*       c1 = COS(alpha)
*       U3(1) = a1 * SU3(1) + b1 * SV3(1) + c1 * grad3(1)
*       U3(2) = a1 * SU3(2) + b1 * SV3(2) + c1 * grad3(2)
*       U3(3) = a1 * SU3(3) + b1 * SV3(3) + c1 * grad3(3)
*       CALL renorm(U3)
*       COL6(5)=COL6(5)+1.0
*
*------------------------------------------------------------------------*
*  EXAMPLE 4, if incidence angle > 60 ==> mirror-like, no sticking       *
*      Otherwise, cosine law reflection + maxwell-Boltzmann thermali-    *
*      zation + sticking time ~ exp(-t/t0), t0=1E-5s                     *
*------------------------------------------------------------------------* 
*
*       zero=0.0
*       cosphi = V3scaV3(U3,grad3)/(norm(U3)*norm(grad3))  ! cos(phi)=a * b / |a|*|b|  
*       IF (cosphi.lt.0.5) THEN                            ! (phi>60\deg)
*         k = V3scaV3(grad3,U3) ! k = grad3(1)*U3(1)+grad3(2)*U3(2)+grad3(3)*U3(3)
*         U3(1) = U3(1) - 2 * k * grad3(1)                 
*         U3(2) = U3(2) - 2 * k * grad3(2)
*         U3(3) = U3(3) - 2 * k * grad3(3)
*         COL6(5) = COL6(5) + 1.0
*       ELSE                                               ! (phi>60\deg)
*         CALL cosineLaw(grad3,U3)                         ! existing function 
*         CALL Boltzmann(U3,T,A,mode)                      ! existing function 
*C- - - - - Now adding updating total desorption time per particle, tde:
*         zero=0.0
*         tde = tde + ts * log(1/rand(zero))               ! ts provided at runtime, e.g. 1E-6 [s]
*         COL6(6) = COL6(6) + 1.0                
*       END IF
*------------------------------------------------------------------------*
*  EXAMPLE 5, Sample the VELOCITY vector (angle,speed) with the cosine   *
*      Law and and M-B thermalization.                                   *
*      USE customized STICKING, as a function of position, X3, surface   * 
*      and with a distribution function (do not just use average value)  *
*      of the type : exp(-t/ts)/ts, ts=1E-5s                             *
*      The dependence with position is, e.g.:  t0 = ts * (1 + 1.25*z)    *
*      The surface (surn = 5) is a COLD SPOT where: t0 = ts * 100.0      * 
*------------------------------------------------------------------------* 
*
*       zero=0.0
*       CALL cosineLaw(grad3,U3)                           ! existing function 
*       CALL Boltzmann(U3,T,A,mode)                        ! existing function 
*C- - - - - Now adding updating total desorption time per particle, tde:
*       IF (surn.eq.5) THEN
*         t0 = 100.0 * ts        
*       ELSE
*         t0 = ts * ( 1 + 1.25 * X3(3))
*       END IF
*       tde = tde + t0 * log(1/rand(zero))                 ! Frenkel Law
*       COL6(1) = COL6(1) + 1.0                            ! CosineLaw type counter
**      WARNING: use a positive ts, with a negative time you would be using twice the exponential law!
      END
