      SUBROUTINE customsource(X3,U3,tp,SOURCE)
        real*8 X3(3),U3(3),tp                                            ! i/o
        real*8 SOURCE(14)                                                ! i
        real*8 a,b,zero,pi                                               ! a
        integer*4 i,j,k                                                  ! a
        external rand                                                    ! e  
**************************************************************************
*               CUSTOMIZE THIS FUNCTION TO FIT YOUR PROBLEM              *
*   USAGE:  special sources for the isotope generation (e.g. to fit the  *
*             radioactive x-t production computed with FLUKA)            *
*           This function is always called by the InitPar subroutine     *
*                                                                        *
*   VARIABLES:                                                           *
*               INPUT                                                    *
*     SOURCE(14) ---  Parameters of the starting particle given in the   *
*                        input file, card "Source". This parameters are  *
*                        used to define standart sources, but their      *  
*                        values can also be used as input for other      *
*                        sources defined by the user. Their meaning is:  *
*                          1: A (Mass number)                            *
*                          2: T (Initial Energy [K])                     *
*                          3: alpha (Semiangle of isotropy cone for the  *
*                                     initial direction)                 *
*                          4,5,6: nz,ny,nz (Axis of cone for the inital  *
*                                     direction of the speed)            *
*                          7,8,9: x0,y0,z0 (Centroid of initial          *
*                                     position [cm])                     *
*                     For Tally identifier 'P' (Point source)            *
*                          - No more values                              *
*                     For Tally identifier 'B' (Box)                     *
*                          10,11,12: Lx,Ly,Lz (full length of the 3      *
*                                     sides of the box [cm]              * 
*                          13,14: thita,phi (Orientation of the box)     *
*                     For Tally identifier 'T' (Target)                  *
*                          10:R (Radius of cylinder of generation [cm])  *
*                          11:L (Length of cylinder of generation [cm])  *
*                          12: Sigma (Beam sigma [cm])                   *
*                          13,14: thita,phi (Orientation of the cylinder)*   
*         INPUT/OUTPUT                                                   *
*     X3(3)  -------  Absolute position at generation point (x,y,z) [cm] *
*     U3(3)  -------  Starting velocity {ux,uy,uz} of the atom [m/s]     *
*                        NOTE: the module of the vector is previously    *
*                        computed from the Maxwell-Boltzmann distribu-   *
*                        tion, for T, A (SOURCE(1-2)                     *
*                        The direction is previously sampled with values *
*                        SOURCE(3-6).                                    *
*                        The user CAN modify U3                          *
*     tp      ------  Particle flight time [s]. Normally it should be    *
*                        initialised to 0.0, if everything is created    *
*                        at the beginning. Otherwise, you can introduce  * 
*                        the pulse structure here                        *
*           AUXILIARY                                                    *
*     a,b,pi,...----  auxiliary variables (can add more)                 *
*     ...                                                                *
*            EXTERNAL                                                    *
*     rand   -------  Seed for random generation                         *
*                                                                        *
*   NOTES:                                                               *
*     - Check available mathematical functions in math.f                 *
*     -                                                                  *
**************************************************************************
*
*
**************************************************************************
*
*        zero=0.0
*        pi=3.141592654
*
*------------------------------------------------------------------------*
**   EXAMPLE 1: sample a certain function. Use inverse transform         *
**      remember that you are SAMPLING: you must suply the probability   *
**      function in terms of a random number ...   Conditions:           *
**         -  x uniform between xmin and xmax, i.e.                      *                                                  
**            SOURCE(11) = xmaxn - xmax = L                              *
**         -  y is a function of x, e.g. y=sin(pi*x/L)                   * 
*------------------------------------------------------------------------*
*        X3(1) = SOURCE(11)*rand(zero)-(SOURCE(11)/2)+SOURCE(7)
*        X3(2) = sin(pi*X3(1)/SOURCE(11))
*        X3(3) = 0.0
**************************************************************************
**   Note that you can also sample the speeds ux,uy,uz                   *
**   you can use the mass (SOURCE(1)) and the energy (SOURCE(2))         *   
**************************************************************************
*
*------------------------------------------------------------------------*
**   EXAMPLE 2: read data from a data/init.dat file and store it in the  *
**       starting 6-space coordinates + time                             *
*------------------------------------------------------------------------*
*        read(9,*)X3,U3,tp
*
**************************************************************************
**   remember your custom source is compatible with the cell exclusive   *
**   source (source limited to a cell, runtime option                    * 
**************************************************************************
**     
*------------------------------------------------------------------------*
**   EXAMPLE 3: SPES target. 7 UC pills with decaying production         *
**   use in conjugation with Target Source card of input file (spes.t):  *
**   type  Mass  T(K)   Alpha   nx  ny   nz  x	y     z   R    L   sigma  thita  phi
**    T     12    2000   180     0   1    0   0	8.50  0   3    13   0.5    90    90  
**   Generation in 7 thin pills dictated by the following law:           *
**       pill  position_y  production  probability Probability           *
**        1       2.08       2.85	       0.19        0.19          *
**        2       4.22       2.63	       0.17	   0.36          *
**        3       6.36       2.41	       0.16        0.51          *
**        4       8.49       2.2	       0.14	   0.66          *
**        5      10.61       1.98	       0.13	   0.78          *
**        6      12.73       1.76	       0.11	   0.9           *
**        7      14.83       1.55	       0.1	   1.0           *
**   SOLUTION:                                                           *
*------------------------------------------------------------------------*
*        a=rand(zero)    
*        IF (a.gt.0.9) THEN
*          y=14.83
*        ELSE IF (a.gt.0.78) THEN
*          y=12.73
*        ELSE IF (a.gt.0.66) THEN
*          y=10.61
*        ELSE IF (a.gt.0.51) THEN
*          y= 8.49
*        ELSE IF (a.gt.0.36) THEN
*          y= 6.36
*        ELSE IF (a.gt.0.19) THEN
*          y= 4.22
*        ELSE
*          y= 2.08
*        END IF
      END
