      SUBROUTINE surfaceBase(SU3,SV3,SW3)
        real*8 SW3(3)                                             ! i
        real*8 SU3(3),SV3(3)                                      ! o
        CALL renorm(SW3)                                          ! making sure to have unitary vectors
        IF (abs(SW3(1)).gt.1E-6.or.abs(SW3(2)).gt.1E-6) THEN
          SU3(1) = SW3(2) 
          SU3(2) = -SW3(1)
          SU3(3) = 0.0
        ELSE
          SU3(1) = 0.0
          SU3(2) = SW3(3)
          SU3(3) = -SW3(2)  
        END IF
        CALL renorm(SU3)
        CALL V3vecV3(SW3,SU3,SV3)                                 ! SV3 = SW3 ^ SU3
      END


      FUNCTION norm(U3)
        real*8 U3(3)                                              ! i
        real*8 norm                                               ! o
        norm = sqrt(U3(1)**2 + U3(2)**2 + U3(3)**2)
      END

      SUBROUTINE renorm(U3)
        real*8 U3(3)                                              ! i/o
        real*8 norm,mod                                           ! a
        mod = norm(U3)
        U3(1) = U3(1) / mod
        U3(2) = U3(2) / mod
        U3(3) = U3(3) / mod
      END

      SUBROUTINE V3prodNUM(U3,v)                                   
        real*8 v                                                  ! i
        real*8 U3(3)                                              ! i/o
        U3(1) = U3(1) * v
        U3(2) = U3(2) * v
        U3(3) = U3(3) * v
      END

      SUBROUTINE V3vecV3(A3,B3,C3)
        real*8 C3(3)                                              ! o
        real*8 A3(3),B3(3)                                        ! i
        C3(1) = A3(2) * B3(3) - A3(3) * B3(2)
        C3(2) = A3(3) * B3(1) - A3(1) * B3(3)
        C3(3) = A3(1) * B3(2) - A3(2) * B3(1)
      END

      SUBROUTINE invertV(U3)
        real*8 U3(3)                                              ! i/o
        U3(1) = -U3(1)
        U3(2) = -U3(2)
        U3(3) = -U3(3)
      END

      FUNCTION V3scaV3(A3,B3)
        integer*4 i                                               ! a
        real*8 A3(3),B3(3),sum                                    ! i/a
        real*8 V3scaV3                                            ! o
        sum = 0.0
        i = 1
        DO WHILE (i.le.3)
          sum = sum + A3(i) * B3(i)
          i = i + 1
        END DO
        V3scaV3 = sum
      END

      SUBROUTINE cosineLaw(SW3,U3)  ! SW3 = central direction --> U3 (within 2Pi erad)
       real*8 SW3(3)                                                      ! i
       real*8 U3(3)                                                       ! o
       real*8 zero,beta,sinalpha,cosalpha,r,SU3(3),SV3(3),a,b,c,Pi        ! a
       integer*4 i                                                        ! a
       real*8 rand                                                        ! e
       external rand                                                      ! e
       zero=0.0
       Pi=3.1415927
       r=rand(zero)
       cosalpha = sqrt(r)
       sinalpha = sqrt(1-r)
       beta = 2*Pi*rand(zero)
       CALL surfaceBase(SU3,SV3,SW3)
       a = sinalpha * COS(beta)   
       b = sinalpha * SIN(beta)    
       c = cosalpha                
       i=1
       DO WHILE (i.le.3)
         U3(i) = (a * SU3(i)) + (b * SV3(i)) + (c * SW3(i))
         i=i+1
       END DO
       CALL renorm(U3)
      END


      SUBROUTINE isotropic(SW3,U3,pangle)
       real*8 SW3(3),pangle                                               ! i
       real*8 U3(3)                                                       ! o
       real*8 zero,beta,sinalpha,cosalpha,r,SU3(3),SV3(3),a,b,c,Pi        ! a
       integer*4 i
       real*8 rand
       external rand
       zero=0.0
       Pi=3.1415927
       r= ((1-cos(pangle*Pi/180.0))*rand(zero)) + cos(pangle*Pi/180.0)    ! if pangle=Pi/2 --> r=rand(zero)
       cosalpha = r
       sinalpha = sqrt(1-(r**2))
       beta = 2*Pi*rand(zero)
       CALL surfaceBase(SU3,SV3,SW3)
       a = sinalpha * COS(beta)   
       b = sinalpha * SIN(beta)    
       c = cosalpha 
       i=1
       DO WHILE (i.le.3) 
         U3(i) = a * SU3(i) + b * SV3(i) + c * SW3(i)
         i=i+1
       END DO
       CALL renorm(U3)
      END


      SUBROUTINE gaussian(g,mu,sigma)
        real*8 g                                                  ! o
        real*8 mu,sigma                                           ! i
        real*8 sum,zero                                           ! a
        integer*4 i                                               ! a
        real*8 rand
        external rand                                             ! e
        zero = 0.0
        i = 1
        sum = 0.0
        DO WHILE (i.le.12)    ! central limit theorem up to 12th term
          sum = sum + rand(zero)
          i = i + 1
        END DO
        g = mu + sigma * (sum - 6)
      END
