*
**************************************************************************
*             readEfield is a subroutine of the RIBO code                *
**************************************************************************
*
**************************************************************************
*    RRRR   EEEEE   AAA   DDDD   EEEEE  FFFFF  I  EEEEE  L      DDDD     *
*    R   R  E      A   A  D   D  E      F      I  E      L      D   D    *
*    RRRR   EEE    AAAAA  D   D  EEEE   FFFF   I  EEEE   L      D   D    *
*    R  R   E      A   A  D   D  E      F      I  E      L      D   D    *
*    R   R  EEEEE  A   A  DDDD   EEEEE  F      I  EEEEE  LLLLL  DDDD     *
**************************************************************************
*        THIS ROUTINE READS ELECTRIC FIELD DATA (SI) FROM A FILE         *
*                        and/or from constants                           * 
**************************************************************************
*                                                                        *
*     DEPENDENCIES:                                                      *
*                                                                        *
*                  checkCELL--|--->--------------|--------------|        *
*     __________              |                                 |        *
*     readEfield |----> El ---|                                 |        *
*     ~~~~~~~~~~              |---> computedt ---|              |        *
*         Bn ----|            |                  |--> EMfield --|- main  *
*                |----> B ----|                  |                       *
*                |---->-------|---> electrans ---|                       *
*                             |                                          *
*                 backupstep--|                                          *
*                                                                        *
**************************************************************************
*                                                                        *
*     VARIABLES:                                                         *
*                                                                        *
*     *) x,y,z: absolute coordiantes,  [x,y,z]=m                         * 
*     *) E(3):  electric field vector (SI)                               *
*                                                                        *
**************************************************************************
*
      SUBROUTINE readEfield(x,y,z,E)
        real*8 x,y,z,E(3)
*
*
*********************          example         ***************************
*                EXTRACTION FIELD + PLASMA RADIAL FIELD                  *
*------------------------------------------------------------------------*
**   A field E={Er,Ey} penetrates a tube up to Length-Upen = 4.5 cm = y  *
**   The tube has a radius Rmax = 2 cm                                   *
**   We have a map of the electrostatic potential U for a grid with      *
**     17 radial divisions and 68 longitudinal divisions --> Ury(17,68)  *
**   There is a second radial field due to plasma effects Up             *
**     this field behaves as Up(r)=Up*exp(-r/lambda)/(lambda*0.01)       *
*------------------------------------------------------------------------*
*        real*8 Ury(17,68),r,Rmax,Er                                     *
*        real*8 Ex,Ey,Ez,Up,lambda                                       *
*        integer*4 i,j                                                   * 
***                                                                      *  
*        r=x*x+y*y                                                       *
*        Ymax=5                                                          *
*        Rmax=2                                                          *
*        Udep=0.5                                                        *
*        IF (y.ge.(Ymax-Udep).and.r.lt.Rmax) THEN                        *
*          open(UNIT=1,FILE='data/Urz.dat',STATUS='OLD')                 *
*          read(1,*)Ury                                                  *
*          close(1)                                                      *
*          Ey=30.0*100.0*(Ury(i,j)-Ury(i,j+1))/(Udep/67.0)               *
*          Er=30.0*100.0*(Ury(i,j)-Ury(i+1,j))/(Rmax/15.0)               *
*        END IF                                                          *
*** need to define Up and lambda (see Koe2000)                           *
*        Er=Er+(Up*exp(-r/lambda)/(lambda*0.01))                         *
*        Ey=Ey                                                           *
*** conversion to rectangular coordinates                                *
*        Ex=Er*x/(abs(r+1E-10))                                          *
*        Ez=Er*z/(abs(r+1E-10))                                          *
*        E(1)=Ex                                                         *
*        E(2)=Ey                                                         *
*        E(3)=Ez                                                         *
**************************************************************************
*
        E(1)=0.0
        E(2)=0.0
        E(3)=0.0
      END
       
